<?php

namespace Drupal\disable_language;


use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Language\LanguageDefault;
use Drupal\language\Config\LanguageConfigFactoryOverrideInterface;
use Drupal\language\ConfigurableLanguageManager;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Language\LanguageManager;

/**
 * Class DisableLanguageManager
 * @package Drupal\disable_language
 */
class DisableLanguageManager extends ConfigurableLanguageManager {

  /**
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * @var Enabled languages
   */
  protected $enabledLanguages;

  /**
   * @var Disabled languages
   */
  protected $disabledLanguages;

  /**
   * @var \Drupal\Core\Language\LanguageManager
   */
  protected $languageManager;


  /**
   * DisableLanguageManager constructor.
   * @param \Drupal\Core\Language\LanguageDefault $default_language
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   * @param \Drupal\language\Config\LanguageConfigFactoryOverrideInterface $config_override
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   * @param \Drupal\Core\Entity\EntityTypeManager $entityTypeManager
   * @param \Drupal\Core\Language\LanguageManager $languageManager
   */
  public function __construct(LanguageDefault $default_language, ConfigFactoryInterface $config_factory, ModuleHandlerInterface $module_handler, LanguageConfigFactoryOverrideInterface $config_override, RequestStack $request_stack, EntityTypeManager $entityTypeManager, languageManager $languageManager) {
    parent::__construct($default_language, $config_factory, $module_handler, $config_override, $request_stack);
    $this->entityTypeManager = $entityTypeManager;
    $this->languageManager = $languageManager;
  }


  /**
   * @return array|\Drupal\Core\Entity\EntityInterface[]
   */
  public function getConfigurableLanguages() {
    $configurableLanguages = [];

    // Get all languages
    $languages = $this->getLanguages();

    //The language itself doesn't own the thirdPartySetting,
    //So we need to use its matching ConfigEntity
    //Getting the ConfigurableLanguageManager
    $configManager = $this->entityTypeManager->getStorage('configurable_language');

    $configurableLanguages = $configManager->loadMultiple(array_keys($languages));

    return $configurableLanguages;

  }

  /**
   * @return mixed
   */
  public function getDisabledLanguages() {

    if (empty($this->disabledLanguages)) {
      foreach ($this->getConfigurableLanguages() as $langcode => $configurableLanguage) {
        $disabled = $configurableLanguage->getThirdPartySetting('disable_language', 'disable');
        if (isset($disabled) && $disabled == 1) {
          $this->disabledLanguages[$langcode] = $configurableLanguage;
        }
      }
    }
    return $this->disabledLanguages;
  }

  /**
   * @return mixed
   */
  public function getEnabledLanguages() {

    if (empty($this->enabledLanguages)) {
      foreach ($this->getConfigurableLanguages() as $langcode => $configurableLanguage) {

        $disabled = $configurableLanguage->getThirdPartySetting('disable_language', 'disable');
        if ((isset($disabled) && $disabled == 0) || empty($disabled)) {
          $this->enabledLanguages[$langcode] = $configurableLanguage;
        }

      }
    }
    return $this->enabledLanguages;
  }

  /**
   * @return bool
   */
  public function isCurrentLanguageDisabled() {

    $currentLanguageDisabled = FALSE;

    $disabledLangCodes = array_keys($this->getDisabledLanguages());

    if (isset($disabledLangCodes) && in_array($this->languageManager->getCurrentLanguage()
        ->getId(), $disabledLangCodes)
    ) {
      $currentLanguageDisabled = TRUE;
    }

    return $currentLanguageDisabled;
  }

  /**
   * @return \Drupal\core\Language\LanguageInterface|null
   */
  public function getFirstEnabledLanguage() {
    $enabledLanguages = $this->getEnabledLanguages();
    $configurableLanguage = reset($enabledLanguages);
    return $this->getLanguage($configurableLanguage->getId());
  }

}