<?php

namespace Drupal\disable_language\EventSubscriber;

// This is the interface we are going to implement.
use Drupal\Core\Session\AccountProxy;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
// This class contains the event we want to subscribe to.
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\Event\GetResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;
use Drupal\Core\Url;
use Drupal\disable_language\DisableLanguageManager;

/**
 * Subscribe to KernelEvents::REQUEST events and redirect if site is currently
 * in maintenance mode.
 */
class DisabledLanguagesEventSubscriber implements EventSubscriberInterface {
  /**
   * {@inheritdoc}
   */

  /**
   * @var \Drupal\disable_language\DisableLanguageManager
   */
  protected $disableLanguageManager;

  /**
   * @var \Drupal\Core\Session\AccountProxy
   */
  protected $currentUser;

  /**
   * DisabledLanguagesEventSubscriber constructor.
   * @param \Drupal\disable_language\DisableLanguageManager $disableLanguageManager
   * @param \Drupal\Core\Session\AccountProxy $currentUser
   */
  public function __construct(DisableLanguageManager $disableLanguageManager, AccountProxy $currentUser) {
    $this->currentUser = $currentUser;
    $this->disableLanguageManager = $disableLanguageManager;
  }

  /**
   * @return mixed
   */
  static function getSubscribedEvents() {
    // On a normal request
    $events[KernelEvents::REQUEST][] = array('checkForDisabledLanguageAndRedirect');
    // On an access denied request
    $events[KernelEvents::EXCEPTION][] = array(
      'checkForDisabledLanguageAndRedirect',
      0
    );
    return $events;
  }

  /**
   * @param \Symfony\Component\HttpKernel\Event\GetResponseEvent $event
   */
  public function checkForDisabledLanguageAndRedirect(GetResponseEvent $event) {

    // Only redirect if the user doesn't have access to view disabled languages
    if (!$this->currentUser->hasPermission('view disabled languages')) {
      if ($this->disableLanguageManager->isCurrentLanguageDisabled()) {

        $language = $this->disableLanguageManager->getFirstEnabledLanguage();
        $url = Url::fromRoute('<front>', [], ['language' => $language]);
        $response = new RedirectResponse($url->toString(), '307');
        $event->setResponse($response);
      }
    }
  }
}
